package nckd.poc602.plugin.task;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;

import kd.bos.context.RequestContext;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.metadata.dynamicobject.DynamicObjectType;
import kd.bos.dataentity.serialization.SerializationUtils;
import kd.bos.entity.EntityMetadataCache;
import kd.bos.entity.MainEntityType;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.exception.KDException;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.schedule.executor.AbstractTask;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;

/**
 * Module           :供应链-合同管理
 * Description      :采购合同未进行价格审定提醒
 * @author : xiaobing_wu
 * @date : 2025-4-26
 */
public class PurContractHaveNotJGSDTask extends AbstractTask {
	protected static Log logger = LogFactory.getLog(PurContractHaveNotJGSDTask.class);
    @Override
    public void execute(RequestContext requestContext, Map<String, Object> map) throws KDException {
        //1、从销售合同变跟单中查询所有已审核&变更原因=A：价格审定，获取所有
    	Set<Object> xshtbillnos = new HashSet<>();
        QFilter qxsbg = new QFilter("billstatus", QCP.equals, "C");
        qxsbg.and("nckd_combofield",QCP.equals,"A");                  
        DynamicObject collections[] = BusinessDataServiceHelper.load("conm_xsalcontract","billno", qxsbg.toArray());
        for (DynamicObject saleData : collections) {
            Object billno = saleData.get("billno");
            xshtbillnos.add(billno);
        }
        logger.info("PurContractHaveNotJGSDTask-xshtbillnos:" + xshtbillnos);
        if (!xshtbillnos.isEmpty()) {
        	Map<String,String> xsht2cghtbillno = new HashMap<>();
        	Map<String,Set<String>> xsht2cght = new HashMap<>();
            QFilter qcght = new QFilter("billstatus", QCP.equals, "C");
            qcght.and("nckd_refbillfield.billno", QCP.in, xshtbillnos);
            List<String> cghtbillnos = new ArrayList<>();
            DynamicObjectCollection cghts = QueryServiceHelper.query("conm_purcontract","id,billno,nckd_refbillfield.billno", qcght.toArray(), "");
            for(DynamicObject cght : cghts) {
            	String billno = cght.getString("billno");
            	String xshtbillno = cght.getString("nckd_refbillfield.billno");
            	xsht2cghtbillno.put(xshtbillno, billno);
            	cghtbillnos.add(billno);
            }
            logger.info("PurContractHaveNotJGSDTask-cghtbillnos:" + cghtbillnos);
            logger.info("PurContractHaveNotJGSDTask-xsht2cghtbillno:" + xsht2cghtbillno);
            if(!cghtbillnos.isEmpty()) {
            	QFilter qcghtbg = new QFilter("billstatus", QCP.equals, "C");
            	qcghtbg.and("billno", QCP.in, cghtbillnos);
            	qcghtbg.and("nckd_combofield",QCP.equals,"A");
                DynamicObjectCollection cghtbgs = QueryServiceHelper.query("conm_xpurcontract","id,billno", qcghtbg.toArray(), "");
                if(!cghtbgs.isEmpty()){
                    for (DynamicObject cghtbg : cghtbgs) {
                        String billno = cghtbg.getString("billno");//采购合同编号
                        cghtbillnos.remove(billno);
                    }
                }
                logger.info("PurContractHaveNotJGSDTask-cghtbillnos2:" + cghtbillnos);
            }
            logger.info("PurContractHaveNotJGSDTask-cghtbillnos3:" + cghtbillnos);
            StringBuffer bf = new StringBuffer();
            if(!cghtbillnos.isEmpty()) {
            	
            	for(String cghtbillno : cghtbillnos) {
            		String xshtbillno = xsht2cghtbillno.get(cghtbillno);
            		Set<String> cgs = xsht2cght.get(xshtbillno);
            		if(cgs == null) {
            			cgs = new HashSet<>();
            			cgs.add(cghtbillno);
            			xsht2cght.put(xshtbillno, cgs);
            		}else {
            			cgs.add(cghtbillno);
            		}
            	}
            	logger.info("PurContractHaveNotJGSDTask-xsht2cght:" + xsht2cght);
            	
            	for(String xsht : xsht2cght.keySet()) {
            		Set<String> cght = xsht2cght.get(xsht);
            		String tm = "以下采购合同（来源于销售合同：" + xsht + "）未进行价格审定";
            		bf.append(tm);
            		bf.append(System.lineSeparator());
            		bf.append(cght.toString());
            		bf.append(System.lineSeparator());
            	}
            	logger.info("PurContractHaveNotJGSDTask-bf:" + bf);
            	if(StringUtils.isNotBlank(bf.toString())) {
            		this.buildBGYJD(bf.toString());
            	}
            }	
        }
    }
    
    private void buildBGYJD(String error) {
    	logger.info("PurContractHaveNotJGSDTask-buildBGYJD");
		String entityNumSupplier = "nckd_cght_bg_yjd";
		DynamicObject yjd = BusinessDataServiceHelper.newDynamicObject(entityNumSupplier);
		//结算组织
		yjd.set("nckd_error", error);
		
	    List<DynamicObject> yjds = new ArrayList<>();
	    yjds.add(yjd);
		//保存单据
		OperationResult sRet = OperationServiceHelper.executeOperate("save","nckd_cght_bg_yjd",yjds.toArray(new DynamicObject[yjds.size()]),OperateOption.create());
		if(sRet.isSuccess()) {
			MainEntityType dt = EntityMetadataCache.getDataEntityType("nckd_cght_bg_yjd");
			List<Object> yjdids = sRet.getSuccessPkIds();
			DynamicObject[] yjdobs = BusinessDataServiceHelper.load(yjdids.toArray(), (DynamicObjectType)dt);
			sRet = OperationServiceHelper.executeOperate("submit","nckd_cght_bg_yjd",yjdobs,OperateOption.create());
			if(sRet.isSuccess()) {
				yjdids = sRet.getSuccessPkIds();
				yjdobs = BusinessDataServiceHelper.load(yjdids.toArray(), (DynamicObjectType)dt);
				sRet = OperationServiceHelper.executeOperate("audit","nckd_cght_bg_yjd",yjdobs,OperateOption.create());
			}
		}
		logger.info("PurContractHaveNotJGSDTask-isSuccess：" + sRet.isSuccess());
		logger.info("PurContractHaveNotJGSDTask-getMessage：" + sRet.getMessage());
		
		logger.info("PurContractHaveNotJGSDTask-getAllErrorInfo：" + SerializationUtils.toJsonString(sRet.getAllErrorInfo()));
    }
}
